
#-*- coding:UTF-8 -*-
# Copyright (C) 2003  Pedro Werneck
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""Base classes for fake http server


"""

# System imports
import time

# HoneyPython imports
from Honey.Protocol import LineProtocol

from apache1 import MAP_ERROR

PATH = "/var/www"

JPG = 'HTTP/1.1 200 OK\n'\
      'Date: %(time)s\n'\
      'Server: %(version)s\n'\
      'Last-Modified: Thu, 14 Aug 2003 00:46:45 GMT\n'\
      'Accept-Ranges: bytes\n'\
      'Content-Length: %(length)s\n'\
      'Connection: close\n'\
      'Content-Type: image/jpeg\n'\

# Constants
HTTP10 = 'HTTP/1.0'
HTTP11 = 'HTTP/1.1'
ROOT = '/'
INDEX = '/index.html'
GET = 'GET'
HEAD = 'HEAD'
POST = 'POST'
VERSION = 'Apache/1.3.27 (Unix) PHP/4.3.2'


METHODS = {HTTP10:[GET, HEAD, POST],
           HTTP11:[GET, HEAD, POST]}



class HTTPRequest(object):
    def __init__(self, method, url, version):
        self.method = method
        self.url = url
        self.version = version
        self.headers = {}

    def addHeader(self, header, value):
        self.headers[header] = value

    def __iter__(self):
        return iter((self.method, self.url, self.version))

    def __str__(self):
        return '%s %s %s'%(self.method, self.url, self.version)


class FakeHTTP(LineProtocol):
    """Define the HTTP Protocol

    """
    request = None
    finished = False
    length = 0
    
    def connectionEstablished(self):
        pass

    def lineReceived(self, line):
        if self.request is None:
            if line:
                self.requestReceived(line)

        elif self.request is not None:
            if line:
                self.headerReceived(line)
            else:
                self.finished = True
                self.processRequest()

    def requestReceived(self, line):
        tokens = line.split()
        count = len(tokens)

        if count == 3:
            method, url, version = tokens
        elif count == 2:
            (method, url), version = tokens, HTTP10
        elif count == 1:
            (method,), url, version = tokens, ROOT, HTTP10
        else:
            self.do_ERROR(400)
            return

        self.request = HTTPRequest(method, url, version.upper())

    def headerReceived(self, line):
        try:
            header, data = line.split(":", 1)
        except ValueError:
            self.server.logDEBUG("sent invalid header")
            return
        header = header.lower()
        data = data.strip()
        if header == 'content-length':
            self.length = int(data)
        self.request.addHeader(header, data)

    def getMethods(self, version):
        try:
            methods = METHODS[version]
        except KeyError:
            s, v = version.split('/', 1)
            v = float(v)
            if v < 1.0:
                version = HTTP10
            elif v > 1.1:
                version = HTTP11
            methods = METHODS[version]
        return methods
        
    def processRequest(self):
        method, url, version = self.request
        methods = self.getMethods(version)

        if method not in methods:
            self.do_ERROR(501, method=method, methods=methods, url=url)

        else:
            call = getattr(self, 'do_' + method)
            call(url, version)


    def do_GET(self, url, version):
        pass
        
    def do_POST(self):
        pass

    def do_HEAD(self):
        pass
        
    def do_ERROR(self, action, **kwds):
        msg = MAP_ERROR.get(action)
        if msg is not None:
            kwds['time'] = time.ctime()
            kwds['version'] = VERSION
            kwds['request'] = self.request
            
            self.sendline(msg%kwds)
        else:
            self.server.logDEBUG("do_ERROR called with action %d and args: %s"%(action, kwds))
        self.close()

    def connectionLost(self):
        pass
        
    
            
            

