
#-*- coding:UTF-8 -*-
# Copyright (C) 2003  Pedro Werneck
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""Base classes for fake pop3 server


"""

# System imports
import time
import sys

# HoneyPython imports
from Honey.Protocol import LineProtocol

AUTHORIZATION = 'AUTH_'
TRANSACTION = 'TRAN_'
UPDATE = 'UPDT_'

FAIL_TIMEOUT = 5


class FakePOP3(LineProtocol):
    """Define the POP3 Protocol

    """
    state = AUTHORIZATION
    user = None
    pswd = None
    fail = False
    
    def sendlines(self, lines):
        """Helper method

        if lines is a string, send ut
        if lines is a tuple or list, send each string

        """
        
        if isinstance(lines, (tuple, list)):
            for line in lines:
                self.sendline(line % self.attrs)
        else:
            self.sendline(lines % self.attrs)

    def connectionEstablished(self):
        """Callback

        called when client connects

        """
        
        self.greetClient()

    def lineReceived(self, line):
        """Callback

        called when client sends data

        """
        
        # ignore line if session failed
        if self.fail:
            return

        # ignore blank lines
        if line.strip() == '':
            return
        
        tokens = line.split(' ', 1)
        if len(tokens) == 1:
            cmd, arg = tokens[0], None
        elif len(tokens) == 2:
            cmd, arg = tokens

        self.attrs['command'] = cmd.lower()

        try:
            m = getattr(self, "do_" + self.state +  cmd.upper())
            m(arg)
        except AttributeError:
            self.do_ERROR(cmd, arg)
        
    def connectionLost(self):
        """Callback

        called when client closes

        """

        self.sendlines(strings.QUIT)
        
    def greetClient(self):
        """Sends greeting to client upon connection

        """
        
        self.sendlines(strings.GREETING)

    def do_ERROR(self, cmd, arg):
        """Error ocurred (invalid command)

        """
        
        self.sendlines(strings.INVALID_CMD)

    def do_AUTH_USER(self, user):
        """USER command at the AUTH phase

        """
        
        if self.user is not None:
            self.do_ERROR('user', user)
            return
            
        if user is None:
            self.sendlines(strings.TOO_FEW_ARGUMENTS)
            return

        self.user = user
        self.attrs['user'] = user
        self.sendlines(strings.USER_OK)

    def checkPassword(self):
        """Check passwords.

        This default implementation always return false, i.e., all
        clients will fail

        """
        
        return False

    def do_AUTH_PASS(self, pswd):
        """PASS command at the AUTH phase

        """
        
        if pswd is None:
            self.sendlines(strings.TOO_FEW_ARGUMENTS)
            return

        self.pswd = pswd
        self.attrs['password'] = pswd
        if not self.checkPassword():
            # set the session as failed...
            self.fail = True
            # and register the callback to be called
            self.server.engine.register(FAIL_TIMEOUT, self.wrongPassword)

    def wrongPassword(self):
        """Callback

        Called by the server at the end of failed auth timeout

        """
        
        self.sendlines(strings.INCORRECT_PASS)
        self.do_QUIT()

    def do_QUIT(self, arg=None):
        """Closes the connection


        """
        self.sendlines(strings.QUIT)
        self.close()

    do_AUTH_QUIT = do_QUIT
    do_TRAN_QUIT = do_QUIT
    do_UPDT_QUIT = do_QUIT
        
            
        
    
            
            

