
# Copyright (C) 2003  Pedro Werneck
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""

"""

# Systems imports
import select
import socket
from errno import EINTR, EBADF

# Exceptions

from Honey.Error import SelectError
from Honey.Socket import BaseSocket

# Global socket lists
_reads = []
_writes = []

# Join both lists
_modes = _reads, _writes

# Aliases
_select = select.select

# Constants
READ = 0
WRITE = 1

class Select(object):
    def _validate(self, socket):
        if not isinstance(socket, BaseSocket):
            raise SelectError("invalid socket: not a BaseSocket instance")
        
        if not hasattr(socket, "fileno"):
            raise SelectError("invalid socket: no fileno")

        if socket.closed:
            raise SelectError("invalid socket: socket is closed")
            
    def register(self, socket, mode):
        self._validate(socket)
        if socket in _modes[mode]:
            raise SelectError("socket is already registered")
        
        _modes[mode].append(socket)

    def unregister(self, socket, mode):
        self._validate(socket)
        if socket not in _modes[mode]:
            raise SelectError("socket not registered")
        
        _modes[mode].remove(socket)

    def checkSockets(self):
        for mode in _modes:
            for socket in mode:
                try:
                    _select([socket], [socket], [socket], 0)
                except: # bad socket found
                    mode.remove(socket)
                    socket.close()

    def clean(self):
        for mode in _modes:
            del mode[:]

    def doSelect(self,
                 # A trick to cache globals in local namespace
                 reads=_reads,
                 writes=_writes,
                 select=_select):

        while 1:
            try:
                r, w, duh = select(reads, writes, [], 0)
                break
            except ValueError:
                # Something invalid has passed... how ?
                self.checkSockets()
            except TypeError:
                # not a socket, or a closed socket
                self.checkSockets()
            except socket.error, reason:
                code, msg = reason.args
                if code == EINTR:
                    # Interrupted call... 
                    return
                if code == EBADF:
                    # Bad socket... check all and prune it...
                    self.checkSockets()
                else:
                    # I'm really don't know what's going on...
                    raise
                
        return (r, w)
 
                    
        


