
# Copyright (C) 2003  Pedro Werneck
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""

"""

# System imports
import sys
sys.path.insert(0, "..")

import unittest

# HoneyPython imports

from Honey.Misc import Autosuper



class TestAutoSuper(unittest.TestCase):
    def testMethodCalling(self):
        """Method calls using obj.__super.method()"""
        class A(object):
            __metaclass__ = Autosuper
            def m(self):
                self.A = True

        class B(A):
            def m(self):
                self.__super.m()
                self.B = True

        class C(A):
            def m(self):
                self.__super.m()
                self.C = True

        class D(C, B):
            def m(self):
                self.__super.m()
                self.D = True

        obj = D()
        obj.m()
        self.assert_(obj.A)
        self.assert_(obj.B)
        self.assert_(obj.C)
        self.assert_(obj.D)

    def testNameCrashing(self):
        """Avoiding name conflicts"""
        class A(object):
            __metaclass__ = Autosuper

        self.assertRaises(ValueError, type, "A", (A,), {})


if __name__ == "__main__":
    unittest.main()

